"""
Credit card detector module.

This module re-exports the CreditCardDetector from patterns for backward compatibility.
"""

from .patterns import CreditCardDetector

__all__ = ["CreditCardDetector"]
